#!/usr/bin/env python
# encoding: utf-8

from pympg.gen.nginx import NginxConfigGenerator
from pympg.gen.main import ConfigGenerator
import questionary
from prompt_toolkit.styles import Style
from .gen.apache import ApacheConfigGenerator
from sys import exit

style = Style(
    [
        ("separator", "fg:#6C6C6C"),
        ("qmark", "fg:#FF9D00 bold"),
        ("question", ""),
        ("selected", "fg:#2e82c7 bg:#d9d9d9"),
        ("pointer", "fg:#FF9D00 bold"),
        ("answer", "fg:#2e9dd1 bold bg:#363636"),
    ]
)


def main():
    generator: str = questionary.select(
        "What do you want to generate?",
        choices=["Apache Config", "Nginx Config"],
        style=style,
    ).ask()

    gen: ConfigGenerator = None

    if generator == "Apache Config":
        apache_path: str = questionary.path(
            "Where is apache2 located?",
            default="/etc/apache2",
            style=style,
        ).ask()
        if apache_path is None:
            print("Failed to generate: missing apache path!")
            exit(1)

        domains: str = questionary.text(
            "What domains to you want to serve? (Seperated by commas)",
            default="example.com",
            style=style,
        ).ask()
        if domains is None:
            print("Failed to generate: missing domain(s)!")
            exit(1)

        web_loc = questionary.path(
            "What is the path of your website files? (Optional)",
            default="",
            style=style,
        ).ask()
        if web_loc is None:
            print("Failed to generate: missing webroot!")
            exit(1)

        uri_to_forward = questionary.text(
            "What URI do you want to proxy to? (Optional)",
            default="",
            style=style,
        ).ask()
        if uri_to_forward is None:
            print("Failed to generate: missing proxy URI!")
            exit(1)

        if apache_path != "" and domains != "":
            gen = ApacheConfigGenerator(
                apache_path=apache_path,
                domains=domains,
                uri_to_forward=uri_to_forward,
                web_root=web_loc,
            )
            gen.generate()
        else:
            print("Failed to generate: missing answers!")
    elif generator == "Nginx Config":
        nginx_path: str = questionary.path(
            "Where is nginx located?",
            default="/etc/nginx",
            style=style,
        ).ask()
        if nginx_path is None:
            print("Failed to generate: missing apache path!")
            exit(1)

        domains: str = questionary.text(
            "What domains to you want to serve? (Seperated by commas)",
            default="example.com",
            style=style,
        ).ask()
        if domains is None:
            print("Failed to generate: missing domain(s)!")
            exit(1)

        web_loc = questionary.path(
            "What is the path of your website files? (Optional)",
            default="",
            style=style,
        ).ask()
        if web_loc is None:
            print("Failed to generate: missing webroot!")
            exit(1)

        uri_to_forward = questionary.text(
            "What URI do you want to proxy to? (Optional)",
            default="",
            style=style,
        ).ask()
        if uri_to_forward is None:
            print("Failed to generate: missing proxy URI!")
            exit(1)

        if nginx_path != "" and domains != "":
            gen = NginxConfigGenerator(
                nginx_path=nginx_path,
                domains=domains,
                uri_to_forward=uri_to_forward,
                web_root=web_loc,
            )
            gen.generate()
        else:
            print("Failed to generate: missing answers!")
    elif generator == "UFW Firewall":
        # TODO: fix this
        """gen = UfwConfigGenerator()

        allowDeny = questionary.select(
            "Would you like to accept or deny a port?",
            choices=["Allow", "Deny"],
            style=style,
        ).ask()
        port = questionary.text(
            "What port would you like to allow/deny?", style=style
        ).ask()
        if allowDeny is None:
            print("Failed to generate: you did not choose to allow/deny a port.")
            exit(1)
        if port is None:
            print("Failed to generate: Missing port")
            exit(1)
        if allowDeny != "" and port != "":
            gen.generate(port, accept=True if allowDeny == "Allow" else False)
        else:
            print("Failed to generate: missing answers!")"""
    else:
        print("Nothing to generate, exiting...")



from urllib.parse import urlparse
import os
from typing import Union, Literal
from .main import ConfigGenerator

APACHE_LOG_DIR = "${APACHE_LOG_DIR}"


class ApacheConfigGenerator(ConfigGenerator):
    def __init__(
        self,
        domains: str = "example.com",
        web_root: Union[bool, str] = False,
        uri_to_forward: Union[Literal[False], str] = False,
        apache_path: str = "/etc/apache2",
    ):
        self.domains = domains.split(",")
        self.web_root = web_root
        self.forward_uri = uri_to_forward
        self.apache_path = apache_path

    def generate(self):

        domain = self.domains[0]
        server_alias = "".join(list([f"{d} " for d in self.domains[1:]]))

        if self.forward_uri and self.forward_uri.strip() == "":
            self.forward_uri = "localhost"

        if self.forward_uri != "" and not urlparse(self.forward_uri).scheme in {
            "http",
            "https",
        }:
            self.forward_uri = f"http://{self.forward_uri}"

        config = (
            "<VirtualHost *:80>\n"
            f"   ServerName {domain}\n"
            f"   ServerAlias {server_alias}\n"
            if server_alias
            else f"    DocumentRoot {self.web_root}\n"
            if self.web_root != ""
            else ""
            f"   ProxyPass / {self.forward_uri}/\n"
            f"   ProxyPassReverse / {self.forward_uri}/\n"
            if self.forward_uri != ""
            else ""
            "    ErrorLog {APACHE_LOG_DIR}/error.log\n"
            "    CustomLog {APACHE_LOG_DIR}/access.log combined\n"
            "</VirtualHost>"
        )
        sites_dir = f"{self.apache_path}/sites-available"
        if not os.path.exists(sites_dir):
            os.makedirs(sites_dir)
        f = open(f"{sites_dir}/{domain}.conf", "w")
        f.write(config)
        f.close()
        self.reload()

    def reload(self):
        # Reload apache
        if os.name == "nt":
            os.system("httpd -k graceful")
        else:
            os.system("a2enmod proxy")
            os.system("a2enmod proxy_http")
            os.system(f"a2ensite {self.domains[0]}")
            os.system("sudo apachectl -k graceful")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pympg', 'pympg.gen']

package_data = \
{'': ['*']}

install_requires = \
['questionary>=1.9.0']

entry_points = \
{'console_scripts': ['pympg = pympg.cli:main']}

setup_kwargs = {
    'name': 'pympg',
    'version': '0.0.6',
    'description': 'Command-line multipurpose generator.',
    'long_description': '# ![Config Icon](https://www.iconfinder.com/icons/1976051/download/png/64)pympg\n\n<!-- ALL-CONTRIBUTORS-BADGE:START - Do not remove or modify this section -->\n\n[![All Contributors](https://img.shields.io/badge/all_contributors-2-orange.svg?style=flat-square)](#contributors-)\n\n<!-- ALL-CONTRIBUTORS-BADGE:END -->\n\nCommand-line multipurpose config generator.\n\n## Example\n\n![Example GIF](./assets/pympg-example.gif)\n\n## Prerequisites\n\n-   Latest version of Python 3\n-   For apache2 config generation, apache2 (and it\'s proxy modules enabled).\n\n## Installation\n\n-   Run `pip3 install pympg` to install this package.\n-   Run `sudo pympg` to open the generation prompt.\n\n## Development Build Instructions\n\n### Prerequisites\n\n-   Python 3\n-   Python poetry\n\n### Building\n\n-   `git clone https://github.com/throw-out-error/pympg.git`\n-   CD into the folder\n-   Run `./build.sh` to build the package and install it.\n-   Run `sudo pympg` to run pympg\n\n## Contributors ✨\n\nThanks goes to these wonderful people ([emoji key](https://allcontributors.org/docs/en/emoji-key)):\n\n<!-- ALL-CONTRIBUTORS-LIST:START - Do not remove or modify this section -->\n<!-- prettier-ignore-start -->\n<!-- markdownlint-disable -->\n<table>\n  <tr>\n    <td align="center"><a href="https://onyxcode.net"><img src="https://avatars1.githubusercontent.com/u/58049576?v=4" width="100px;" alt=""/><br /><sub><b>Dan</b></sub></a><br /><a href="https://github.com/throw-out-error/pympg/commits?author=onyxcode" title="Code">💻</a></td>\n    <td align="center"><a href="https://theoparis.com/about"><img src="https://avatars0.githubusercontent.com/u/11761863?v=4" width="100px;" alt=""/><br /><sub><b>Theo Paris</b></sub></a><br /><a href="https://github.com/throw-out-error/pympg/commits?author=creepinson" title="Documentation">📖</a> <a href="https://github.com/throw-out-error/pympg/commits?author=creepinson" title="Code">💻</a></td>\n  </tr>\n</table>\n\n<!-- markdownlint-enable -->\n<!-- prettier-ignore-end -->\n\n<!-- ALL-CONTRIBUTORS-LIST:END -->\n\nThis project follows the [all-contributors](https://github.com/all-contributors/all-contributors) specification. Contributions of any kind welcome!\n',
    'author': 'Theo Paris',
    'author_email': 'theoparisdesigns@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<3.10',
}


setup(**setup_kwargs)



from re import S
from urllib.parse import urlparse
import os
from typing import Union, Literal
from .main import ConfigGenerator


class NginxConfigGenerator(ConfigGenerator):
    def __init__(
        self,
        domains: str = "example.com",
        web_root: Union[bool, str] = False,
        uri_to_forward: Union[Literal[False], str] = False,
        nginx_path: str = "/etc/nginx",
    ):
        self.domains = domains.split(",")
        self.web_root = web_root
        self.forward_uri = uri_to_forward
        self.nginx_path = nginx_path

    def generate(self):
        domain = self.domains[0]
        server_alias = " ".join(list(self.domains[1:])) or ""

        if self.forward_uri and self.forward_uri.strip() == "":
            self.forward_uri = "localhost"

        if self.forward_uri != "" and not urlparse(self.forward_uri).scheme in {
            "http",
            "https",
        }:
            self.forward_uri = f"http://{self.forward_uri}"

        config = (
            "server {\n"
            f"   listen 80;\n"
            f"   server_name {domain} {server_alias};\n"
        )
        if self.web_root != "":
            config += f"   root {self.web_root};\n"
        config += (
            "   location / {\n"
            f"       proxy_pass {self.forward_uri};\n"
            "       proxy_set_header Host $http_host;\n"
            "    }\n"
            if self.forward_uri != ""
            else ""
        )
        config += "}\n"
        sites_dir = f"{self.nginx_path}/sites-available"
        if not os.path.exists(sites_dir):
            os.makedirs(sites_dir)
        f = open(f"{sites_dir}/{domain}.conf", "w")
        f.write(config)
        f.close()
        # Enable site
        enabled_sites_dir = f"{self.nginx_path}/sites-enabled"
        if not os.path.exists(enabled_sites_dir):
            os.makedirs(enabled_sites_dir)
        if not os.path.exists(f"{enabled_sites_dir}/{domain}.conf"):
            os.link(
                f"{sites_dir}/{domain}.conf",
                f"{enabled_sites_dir}/{domain}.conf",
            )

        self.reload()

    def reload(self):
        # Reload nginx
        if os.name == "nt":
            # Do nothing for windows, may be resolved in the future
            pass
        else:
            os.system("sudo systemctl restart nginx")



from abc import ABC, abstractmethod


class ConfigGenerator(ABC):
    def __init__(self):
        pass

    @abstractmethod
    def generate():
        raise NotImplementedError()

    @abstractmethod
    def reload():
        raise NotImplementedError()
